unit data;

interface

uses
  Windows, Messages, SysUtils, Classes, Registry, Des;

type
  TCharSet = set of char;

type
  TPass = record
            username : string;
            connect  : string;
            password : string;
          end;

  TPPass = ^TPass;

type
  TPassList = class(TList)
    destructor  Destroy; override;
    procedure   ClearAll;
    procedure   AddLogin(var R:TPass);
    procedure   Load;
    procedure   Save;
    function    Password(User,Connect:string; var psw:string):integer;
    function    Crypt(s:string;key:string):string;
    function    DeCrypt(s:string;key:string):string;
  end;

type
  TLoginControl = record
                      ClassName : string;
                      Text      : string;
                      PosX      : integer;
                      PosY      : integer;
                    end;

  TLoginWindowControl = (Server,Login,Password,Connect);

  TLoginWindow = record
              Name      : string;
              ClassName : string;
              AutoLogin : boolean;
              Controls  : array[TLoginWindowControl] of TLoginControl;
             end;

  TPLogin = ^TLoginWindow;

type
  TLoginList = class(TList)
    destructor Destroy; override;
    procedure  ClearAll;
    procedure  AddLogin(var R:TLoginWindow);
    procedure  Load;
    procedure  Save;
  end;

type
  TWindow = record
              hWnd      : integer;
              ClassName : string;
              Text      : string;
              X         : integer;
              Y         : integer;
              PosX      : integer;
              PosY      : integer;
              Kind      : integer;
            end;

  TPWindow = ^TWindow;

type
  TWindowList = class(TList)
    destructor Destroy; override;
    procedure  ClearAll;
    procedure  AddWindow(var R:TWindow);
    procedure  Fill(P:HWND);
  end;

type
  THwndList = class(TList)
    procedure  FillClass(CN:string);
  private
    EnumClassName : string;
  end;

function  SortFn(Item1, Item2: Pointer): Integer;
function  SortFnWnd(Item1, Item2: Pointer): Integer;
function  WordPosition(const N: Integer; const S: string; const WordDelims: TCharSet): Integer;
function  ExtractWord(N: Integer; const S: string; const WordDelims: TCharSet): string;
function  StrToHex(S: string): string;
function  HexToStr(S: string): string;
procedure ReadKeyBool(Reg:TRegistry;S,K:string; var V : boolean; D : boolean);
procedure ReadKeyInt(Reg:TRegistry;S,K:string; var V : integer; D : integer);
procedure ReadKeyStr(Reg:TRegistry;S,K:string; var V:string; D:string);
procedure WriteKeyBool(Reg:TRegistry;S,K:string; V : boolean);
procedure WriteKeyInt(Reg:TRegistry;S,K:string; V : integer);
procedure WriteKeyStr(Reg:TRegistry;S,K:string; V:string);


implementation

const RegRootKey : string = 'SOFTWARE\ENTPASS\';

function WordPosition(const N: Integer; const S: string; const WordDelims: TCharSet): Integer;
var
  Count, I: Integer;
begin
  Count := 0;
  I := 1;
  Result := 0;
  while (I <= Length(S)) and (Count <> N) do begin
    { skip over delimiters }
    while (I <= Length(S)) and (S[I] in WordDelims) do Inc(I);
    { if we're not beyond end of S, we're at the start of a word }
    if I <= Length(S) then Inc(Count);
    { if not finished, find the end of the current word }
    if Count <> N then
      while (I <= Length(S)) and not (S[I] in WordDelims) do Inc(I)
    else Result := I;
  end;
end;

function ExtractWord(N: Integer; const S: string; const WordDelims: TCharSet): string;
var
  I: Integer;
  Len: Integer;
begin
  Len := 0;
  I := WordPosition(N, S, WordDelims);
  if I <> 0 then
    { find the end of the current word }
    while (I <= Length(S)) and not(S[I] in WordDelims) do begin
      { add the I'th character to result }
      Inc(Len);
      SetLength(Result, Len);
      Result[Len] := S[I];
      Inc(I);
    end;
  SetLength(Result, Len);
end;


function StrToHex(s:string): string;
var i : integer;
begin
  Result := '';
  for i := 1 to Length(s) do Result := Result + IntToHex(byte(s[i]),2);
end;

function HexToStr(s:string): string;
var
  l : integer;
  ss,st : string;
begin
  Result := '';
  st := s;
  l := Length(st);
  repeat
    ss := Trim(copy(st,1,2));
    if ss='' then break;
    st := copy(st,3,l);
    Result := Result + char(byte(StrToInt('$'+ss)));
  until false;
end;

function Like(Mask,S:string): boolean;
var
  i : integer;
begin
  Result := false;
  if Mask[1] = '*' then
  begin
    If (Mask = '*') AND (Length(S) > 0) then
    begin
      Result := true;
      exit;
    end;

    for i := 1 to Length(S) do
    begin
      If like (copy(Mask,2,Length(Mask)),copy(S,i,Length(S))) then
      begin
        Result := true;
        Exit;
      end;
    end;
  end else
  begin
    for i := 1 to Length(Mask) do
    begin
      if i>Length(S) then
      begin
        if Mask[i]<>'*' then Exit else
        begin
          Result := true;
          Exit;
        end;
      end else
      begin
        if Mask[i] = '*' then
        begin
          Result := Like(copy(Mask,i,Length(Mask)),
                         copy(S,i,Length(S)));
          Exit;
        end else
        if (Mask[i]<>S[i]) and (Mask[i]<>'?') then exit;
      end;
    end;
    Result := Length(Mask)>=Length(S);
  end;
end;


procedure ReadKeyBool(Reg:TRegistry;S,K:string; var V : boolean; D : boolean);
var
  R : TRegistry;
  ks : string;
begin
  if Assigned(Reg) then R := Reg else R := TRegistry.Create;
  ks := RegRootKey+S;
  if R.OpenKey(ks,true) then
  begin
    try
      V := R.ReadBool(k);
    except
      V := D;
      try
        R.WriteBool(k,V);
      except end;
    end;
    R.CloseKey;
  end;
  if not Assigned(Reg) then R.Free;
end;

procedure ReadKeyInt(Reg:TRegistry;S,K:string; var V : integer; D : integer);
var
  R : TRegistry;
  ks : string;
begin
  ks := RegRootKey+S;
  if Assigned(Reg) then R := Reg else R := TRegistry.Create;
  if R.OpenKey(ks,true) then
  begin
    try
      V := R.ReadInteger(k);
    except
      V := D;
      try
        R.WriteInteger(k,V);
      except end;
    end;
    R.CloseKey;
  end;
  if not Assigned(Reg) then R.Free;
end;

procedure ReadKeyStr(Reg:TRegistry;S,K:string; var V:string; D:string);
var
  R : TRegistry;
  ks : string;
begin
  ks := RegRootKey+S;
  if Assigned(Reg) then R := Reg else R := TRegistry.Create;
  if ks[Length(ks)]='\' then System.Delete(ks,Length(ks),1);
  if R.OpenKey(ks,true) then
  begin
    try
      V := R.ReadString(k);
    except
      V := D;
      try
        R.WriteString(k,V);
      except end;
    end;
    R.CloseKey;
  end;
  if not Assigned(Reg) then R.Free;
end;


procedure WriteKeyBool(Reg:TRegistry;S,K:string; V : boolean);
var
  R : TRegistry;
  ks : string;
begin
  ks := RegRootKey+S;
  if Assigned(Reg) then R := Reg else R := TRegistry.Create;
  if R.OpenKey(ks,true) then
  begin
    try
      R.WriteBool(k,V);
    except end;
    R.CloseKey;
  end;
  if not Assigned(Reg) then R.Free;
end;

procedure WriteKeyInt(Reg:TRegistry;S,K:string; V : integer);
var
  R : TRegistry;
  ks : string;
begin
  ks := RegRootKey+S;
  if Assigned(Reg) then R := Reg else R := TRegistry.Create;
  if R.OpenKey(ks,true) then
  begin
    try
      R.WriteInteger(k,V);
    except end;
    R.CloseKey;
  end;
  if not Assigned(Reg) then R.Free;
end;

procedure WriteKeyStr(Reg:TRegistry;S,K:string; V:string);
var
  R : TRegistry;
  ks : string;
begin
  ks := RegRootKey+S;
  if Assigned(Reg) then R := Reg else R := TRegistry.Create;
  if ks[Length(ks)]='\' then System.Delete(ks,Length(ks),1);
  if R.OpenKey(ks,true) then
  begin
    try
      R.WriteString(k,V);
    except end;
    R.CloseKey;
  end;
  if not Assigned(Reg) then R.Free;
end;


function SortFn(Item1, Item2: Pointer): Integer;
var C1,C2,U1,U2 : string;
begin
  Result := 0;
  C1 := AnsiUpperCase(TPPass(Item1)^.connect);
  C2 := AnsiUpperCase(TPPass(Item2)^.connect);
  U1 := AnsiUpperCase(TPPass(Item1)^.username);
  U2 := AnsiUpperCase(TPPass(Item2)^.username);

  if C1>C2 then Result := 1 else
  if C1<C2 then Result := -1 else
  if U1>U2 then Result := 1 else
  if U1<U2 then Result := -1;
end;

function SortFnWnd(Item1, Item2: Pointer): Integer;
var C1,C2 : string;
begin
  Result := 0;
  C1 := AnsiUpperCase(TPLogin(Item1)^.Name);
  C2 := AnsiUpperCase(TPLogin(Item2)^.Name);

  if C1>C2 then Result := 1 else
  if C1<C2 then Result := -1;
end;

procedure TPassList.ClearAll;
var i : integer;
begin
  for i := 0 to Count-1 do Dispose(TPPass(Items[i]));
  Clear;
end;

destructor TPassList.Destroy;
begin
  ClearAll;
  inherited Destroy;
end;

function TPassList.Crypt(s:string;key:string):string;
var
  N : Longint;
  K : TDESKey;
  C : TDESCipher;
  st : string;
begin
  st := AnsiUpperCase(key) + 'B591F872';
  with TDoubleDWORD(K) do
  begin
    System.Move((@st[1])^,N,4);
    H := N;
    System.Move((@st[5])^,N,4);
    L := N;
  end;
  C := TDESCipher.Create(K);
  Result := StrToHex(C.EncryptedString(s));
  C.Free;
end;

function TPassList.DeCrypt(s:string;key:string):string;
var
  N : Longint;
  K : TDESKey;
  C : TDESCipher;
  st : string;
begin
  st := AnsiUpperCase(key) + 'B591F872';
  with TDoubleDWORD(K) do
  begin
    System.Move((@st[1])^,N,4);
    H := N;
    System.Move((@st[5])^,N,4);
    L := N;
  end;
  C := TDESCipher.Create(K);
  Result := C.DecryptedString(HexToStr(s));
  C.Free;
end;

procedure TPassList.Load;
var
  R : TPass;
  Reg : TRegistry;
  SL : TStringList;
  i : integer;
  s : string;
begin
  Reg := TRegistry.Create;

  {$ifdef MASTERBANK}
   if not Reg.KeyExists(RegRootKey+'Login') then
   begin
     R.username := '*';
     R.connect  := 'DEVL';
     R.password := 'dvl*';
     AddLogin(R);
     R.username := '*';
     R.connect  := 'TEST';
     R.password := 'mst*';
     AddLogin(R);
     Save(FN);
   end;
  {$else}
   if not Reg.KeyExists(RegRootKey+'Login') then
   begin
     R.username := '*';
     R.connect  := 'TEST';
     R.password := 'TEST*';
     AddLogin(R);
     Save;
   end;
  {$endif}
  ClearAll;
  SL := TStringList.Create;

  Reg.OpenKey(RegRootKey+'Login',true);
  Reg.GetKeyNames(SL);
  Reg.CloseKey;

  for i := 0 to SL.Count-1 do
  begin
    FillChar(R,SizeOf(TPass),0);

    ReadKeyStr(Reg,'Login\'+SL[i],'Server',s,'');
    s := AnsiUpperCase(Trim(s));
    if s='' then continue;
    R.connect := s;

    ReadKeyStr(Reg,'Login\'+SL[i],'User',s,'');
    s := Trim(Decrypt(s,R.connect));
    if s='' then continue;
    R.username := s;

    ReadKeyStr(Reg,'Login\'+SL[i],'Password',s,'');
    R.password := Trim(Decrypt(s,R.connect));
    AddLogin(R);
  end;
  SL.Free;
  Reg.Free;
end;


procedure TPassList.Save;
var
  i : integer;
  R : TPass;
  Reg : TRegistry;
  SL : TStringList;
begin
  Sort(@SortFn);
  SL := TStringList.Create;
  Reg := TRegistry.Create;
  Reg.OpenKey(RegRootKey+'Login',true);
  Reg.GetKeyNames(SL);
  Reg.CloseKey;

  for i := 0 to SL.Count-1 do Reg.DeleteKey(RegRootKey+'Login\'+SL[i]);

  for i := 0 to Count-1 do
  begin
    R := TPPass(Items[i])^;
    WriteKeyStr(Reg,'Login\Login'+IntToStr(i),'Server',R.Connect);
    WriteKeyStr(Reg,'Login\Login'+IntToStr(i),'User',Crypt(R.username,R.Connect));
    WriteKeyStr(Reg,'Login\Login'+IntToStr(i),'Password',Crypt(R.password,R.Connect));
  end;
  Reg.Free;
  SL.Free;
end;


function TPassList.Password(User,Connect:string; var psw:string):integer;
var
  i,n : integer;
  U,C : string;
begin
  Result := -1;
  U := AnsiUpperCase(User);
  C := AnsiUpperCase(Connect);

  for i := 0 to Count-1 do
  begin
    if (AnsiUpperCase(TPPass(Items[i])^.username)=U)
    and (AnsiUpperCase(TPPass(Items[i])^.Connect)=C) then
    begin
      Result := i;
      Psw := TPPass(Items[i])^.password;
      Exit;
    end;
  end;

  for i := 0 to Count-1 do
  begin
    if (AnsiUpperCase(TPPass(Items[i])^.username)=U)
    and (Like(AnsiUpperCase(TPPass(Items[i])^.Connect),C)) then
    begin
      Result := i;
      Psw := TPPass(Items[i])^.password;
      Exit;
    end;
  end;

  for i := 0 to Count-1 do
  begin
    if Like(AnsiUpperCase(TPPass(Items[i])^.username),U)
    and Like(AnsiUpperCase(TPPass(Items[i])^.Connect),C) then
    begin
      Result := i;
      Psw := TPPass(Items[i])^.password;
      repeat
        n := Pos('*',Psw);
        if n=0 then break;
        Psw := copy(Psw,1,n-1)+User+copy(Psw,n+1,Length(psw));
      until false;
      Exit;
    end;
  end;
end;

procedure TPassList.AddLogin(var R:TPass);
var
  i : integer;
  s,c : string;
  P : TPPass;
begin
  c := AnsiUpperCase(R.Connect);

  i := Password(R.username,R.connect,s);

  if (i<0)
  or(AnsiUpperCase(R.username)<>
     AnsiUpperCase(TPPass(Items[i])^.username)) then
  begin
    New(P);
    P^ := R;
    P^.connect := c;
    Add(P);
  end else TPPass(Items[i])^.Password := R.Password;
end;


procedure TLoginList.ClearAll;
var i : integer;
begin
  for i := 0 to Count-1 do Dispose(TPLogin(Items[i]));
  Clear;
end;

destructor TLoginList.Destroy;
begin
  ClearAll;
  inherited Destroy;
end;

procedure TLoginList.Load;
var
  R : TLoginWindow;
  Reg : TRegistry;
  SL : TStringList;
  i,n : integer;
  S : string;

procedure FillDefaulTLoginWindowWindows;
begin
  FillChar(R,SizeOf(TLoginWindow),0);
  R.Name      := 'Dial-up Connection';
  R.ClassName := '#32770';
  R.AutoLogin := true;
  R.Controls[Login].ClassName    := 'Edit';
  R.Controls[Login].PosX         := 9;
  R.Controls[Login].PosY         := 6;
  R.Controls[Password].ClassName := 'Edit';
  R.Controls[Password].PosX      := 9;
  R.Controls[Password].PosY      := 8;
  R.Controls[Server].ClassName   := 'ComboBox';
  R.Controls[Server].PosX        := 9;
  R.Controls[Server].PosY        := 4;
  R.Controls[Connect].ClassName  := 'Button';
  R.Controls[Connect].PosX       := 2;
  R.Controls[Connect].PosY       := 12;
  R.Controls[Connect].Text       := '&Connect';
  AddLogin(R);

  FillChar(R,SizeOf(TLoginWindow),0);
  R.Name      := 'Outlook Express Logon';
  R.ClassName := '#32770';
  R.AutoLogin := true;
  R.Controls[Login].ClassName    := 'Edit';
  R.Controls[Login].PosX         := 7;
  R.Controls[Login].PosY         := 6;
  R.Controls[Password].ClassName := 'Edit';
  R.Controls[Password].PosX      := 7;
  R.Controls[Password].PosY      := 8;
  R.Controls[Server].ClassName   := 'Static';
  R.Controls[Server].PosX        := 9;
  R.Controls[Server].PosY        := 4;
  R.Controls[Connect].ClassName  := 'Button';
  R.Controls[Connect].PosX       := 11;
  R.Controls[Connect].PosY       := 11;
  R.Controls[Connect].Text       := 'OK';
  AddLogin(R);

  FillChar(R,SizeOf(TLoginWindow),0);
  R.Name      := 'Delphi database connect dialog';
  R.ClassName := 'TConnectForm';
  R.Controls[Login].ClassName    := 'TEdit';
  R.Controls[Login].PosX         := 3;
  R.Controls[Login].PosY         := 2;
  R.Controls[Password].ClassName := 'TMaskEdit';
  R.Controls[Password].PosX      := 3;
  R.Controls[Password].PosY      := 3;
  R.Controls[Server].ClassName   := 'TComboBox';
  R.Controls[Server].PosX        := 3;
  R.Controls[Server].PosY        := 4;
  R.Controls[Connect].ClassName  := 'TButton';
  R.Controls[Connect].PosX       := 2;
  R.Controls[Connect].PosY       := 6;
  R.Controls[Connect].Text       := 'Connect';
  AddLogin(R);

  FillChar(R,SizeOf(TLoginWindow),0);
  R.Name      := 'RxLoginDialog component';
  R.ClassName := 'TRxLoginForm';
  R.AutoLogin := true;
  R.Controls[Login].ClassName    := 'TEdit';
  R.Controls[Login].PosX         := 1;
  R.Controls[Login].PosY         := 3;
  R.Controls[Password].ClassName := 'TEdit';
  R.Controls[Password].PosX      := 1;
  R.Controls[Password].PosY      := 4;
  R.Controls[Server].ClassName   := 'TComboBox';
  R.Controls[Server].PosX        := 1;
  R.Controls[Server].PosY        := 5;
  R.Controls[Connect].ClassName  := 'TButton';
  R.Controls[Connect].PosX       := 4;
  R.Controls[Connect].PosY       := 1;
  R.Controls[Connect].Text       := 'OK';
  AddLogin(R);

  FillChar(R,SizeOf(TLoginWindow),0);
  R.Name      := 'SQL Navigator 3.15';
  R.ClassName := 'TfrmPasswd';
  R.AutoLogin := true;
  R.Controls[Login].ClassName    := 'TEdit';
  R.Controls[Login].PosX         := 1;
  R.Controls[Login].PosY         := 5;
  R.Controls[Password].ClassName := 'TEdit';
  R.Controls[Password].PosX      := 1;
  R.Controls[Password].PosY      := 6;
  R.Controls[Server].ClassName   := 'TEdit';
  R.Controls[Server].PosX        := 1;
  R.Controls[Server].PosY        := 7;
  R.Controls[Connect].ClassName  := 'TButton';
  R.Controls[Connect].PosX       := 6;
  R.Controls[Connect].PosY       := 1;
  R.Controls[Connect].Text       := 'OK';
  AddLogin(R);

  FillChar(R,SizeOf(TLoginWindow),0);
  R.Name      := 'SQL Navigator 4';
  R.ClassName := 'TfrmLogOn';
  R.AutoLogin := true;
  R.Controls[Login].ClassName    := 'TEdit';
  R.Controls[Login].PosX         := 5;
  R.Controls[Login].PosY         := 4;
  R.Controls[Password].ClassName := 'TEdit';
  R.Controls[Password].PosX      := 5;
  R.Controls[Password].PosY      := 5;
  R.Controls[Server].ClassName   := 'TComboBox';
  R.Controls[Server].PosX        := 5;
  R.Controls[Server].PosY        := 2;
  R.Controls[Connect].ClassName  := 'TBitBtn';
  R.Controls[Connect].PosX       := 3;
  R.Controls[Connect].PosY       := 8;
  R.Controls[Connect].Text       := 'OK';
  AddLogin(R);

  FillChar(R,SizeOf(TLoginWindow),0);
  R.Name      := 'SQL Navigator 4.1';
  R.ClassName := 'TfrmLogOn';
  R.AutoLogin := true;
  R.Controls[Login].ClassName    := 'TEdit';
  R.Controls[Login].PosX         := 3;
  R.Controls[Login].PosY         := 4;
  R.Controls[Password].ClassName := 'TEdit';
  R.Controls[Password].PosX      := 3;
  R.Controls[Password].PosY      := 5;
  R.Controls[Server].ClassName   := 'TComboBox';
  R.Controls[Server].PosX        := 3;
  R.Controls[Server].PosY        := 2;
  R.Controls[Connect].ClassName  := 'TButton';
  R.Controls[Connect].PosX       := 7;
  R.Controls[Connect].PosY       := 7;
  R.Controls[Connect].Text       := 'OK';
  AddLogin(R);

  FillChar(R,SizeOf(TLoginWindow),0);
  R.Name      := 'TOAD 7.2';
  R.ClassName := 'TfrmLogin';
  R.AutoLogin := true;
  R.Controls[Login].ClassName    := 'TEdit';
  R.Controls[Login].PosX         := 8;
  R.Controls[Login].PosY         := 7;
  R.Controls[Password].ClassName := 'TEdit';
  R.Controls[Password].PosX      := 8;
  R.Controls[Password].PosY      := 8;
  R.Controls[Server].ClassName   := 'TComboBox';
  R.Controls[Server].PosX        := 8;
  R.Controls[Server].PosY        := 5;
  R.Controls[Connect].ClassName  := 'TBitBtn';
  R.Controls[Connect].PosX       := 13;
  R.Controls[Connect].PosY       := 12;
  R.Controls[Connect].Text       := 'Ok';
  AddLogin(R);
end;


function FillControl(var W:TLoginControl;s:string) : boolean;
begin
  Result := false;
  FillChar(W,SizeOf(TLoginControl),0);

  if WordPosition(3,S,[' ',#9])=0 then Exit;

  W.ClassName := ExtractWord(1,S,[' ',#9]);
  if W.ClassName='' then Exit;

  try
    W.PosX := StrToInt(ExtractWord(2,S,[' ',#9]))
  except
    W.PosX := 0;
  end;

  try
    W.PosY := StrToInt(ExtractWord(3,S,[' ',#9]));
  except
    W.PosY := 0;
  end;

  n := WordPosition(4,S,[' ',#9]);
  if n>0 then W.Text := Trim(copy(s,n,Length(s)));
  Result := true;
end;

begin
  ClearAll;
  Reg := TRegistry.Create;
  if not Reg.KeyExists(RegRootKey+'Windows') then
  begin
    FillDefaulTLoginWindowWindows;
    Save;
    Exit;
  end;

  SL := TStringList.Create;
  SL.Sorted := true;

  Reg.OpenKey(RegRootKey+'Windows',true);
  Reg.GetKeyNames(SL);
  Reg.CloseKey;

  for i := 0 to SL.Count-1 do
  begin
    ReadKeyStr(Reg,'Windows\'+SL[i],'Name',R.Name,'');

    ReadKeyStr(Reg,'Windows\'+SL[i],'Class',s,'');
    if s='' then continue;
    R.ClassName := s;

    ReadKeyBool(Reg,'Windows\'+SL[i],'AutoLogin', R.AutoLogin,true);

    ReadKeyStr(Reg,'Windows\'+SL[i],'Server',s,'');
    if not FillControl(R.Controls[Server],s) then continue;

    ReadKeyStr(Reg,'Windows\'+SL[i],'Login',s,'');
    if not FillControl(R.Controls[Login],s) then continue;

    ReadKeyStr(Reg,'Windows\'+SL[i],'Password',s,'');
    if not FillControl(R.Controls[Password],s) then continue;

    ReadKeyStr(Reg,'Windows\'+SL[i],'Connect',s,'');
    FillControl(R.Controls[Connect],s);

    AddLogin(R);
  end;
  SL.Free;
  Reg.Free;

  if Count=0 then
  begin
    FillDefaulTLoginWindowWindows;
    Save;
  end;
end;


procedure TLoginList.Save;
var
  i : integer;
  R : TLoginWindow;
  SL : TStringList;
  Reg : Tregistry;

function WndRecStr(W:TLoginControl):string;
begin
  Result := Trim(W.ClassName+' '+IntToStr(W.PosX)+' '+IntToStr(W.PosY)+' '+W.Text);
end;

begin
  SL := TStringList.Create;
  Reg := TRegistry.Create;
  Reg.OpenKey(RegRootKey+'Windows',true);
  Reg.GetKeyNames(SL);
  Reg.CloseKey;

  for i := 0 to SL.Count-1 do Reg.DeleteKey(RegRootKey+'Login\'+SL[i]);

  for i := 0 to Count-1 do
  begin
    R := TPLogin(Items[i])^;
    WriteKeyStr(Reg,'Windows\Window'+IntToStr(i),'Class',R.ClassName);
    WriteKeyStr(Reg,'Windows\Window'+IntToStr(i),'Name',R.Name);
    WriteKeyBool(Reg,'Windows\Window'+IntToStr(i),'AutoLogin',R.AutoLogin);
    WriteKeyStr(Reg,'Windows\Window'+IntToStr(i),'Server',WndRecStr(R.Controls[Server]));
    WriteKeyStr(Reg,'Windows\Window'+IntToStr(i),'Login',WndRecStr(R.Controls[Login]));
    WriteKeyStr(Reg,'Windows\Window'+IntToStr(i),'Password',WndRecStr(R.Controls[Password]));
    WriteKeyStr(Reg,'Windows\Window'+IntToStr(i),'Connect',WndRecStr(R.Controls[Connect]));
  end;
  Reg.Free;
  SL.Free;
end;


procedure TLoginList.AddLogin(var R:TLoginWindow);
var
  P : TPLogin;
begin
  New(P);
  P^ := R;
  Add(P);
end;


procedure TWindowList.ClearAll;
var i : integer;
begin
  for i := 0 to Count-1 do Dispose(TPWindow(Items[i]));
  Clear;
end;


destructor TWindowList.Destroy;
begin
  ClearAll;
  inherited Destroy;
end;

procedure TWindowList.AddWindow(var R:TWindow);
var
  P : TPWindow;
begin
  New(P);
  P^ := R;
  Add(P);
end;


function EnumWindowProc(WinHandle: HWnd; Param: LongInt): Boolean; stdcall;
var
  i : integer;
  W : TWindow;
  R : TRect;
  Buf : array[1..500] of char;
  res : longint;
begin
  Result := true;
  FillChar(W,SizeOf(TWindow),0);
  W.Kind := -1;

  W.hWnd := WinHandle;

  Buf[1] := #0;

  SendMessageTimeout(WinHandle,WM_GETTEXT,499,integer(@Buf[1]),SMTO_ABORTIFHUNG,1,res);

//  SendMessage(WinHandle,WM_GETTEXT,499,integer(@Buf[1]));
  W.Text := Trim(StrPas(@Buf[1]));

  Buf[1] := #0;
  i := GetClassName(WinHandle,@Buf[1],500);
  if i>0 then W.ClassName := StrPas(@Buf[1]);

  GetWindowRect(WinHandle,R);
  W.X := R.Left;
  W.Y := R.Top;

  TWindowList(Param).AddWindow(W);
end;

function SortFnX(Item1, Item2: Pointer): Integer;
begin
  Result := 0;
  if TPWindow(Item1)^.X>TPWindow(Item2)^.X then Result := 1 else
  if TPWindow(Item1)^.X<TPWindow(Item2)^.X then Result := -1;
end;

function SortFnY(Item1, Item2: Pointer): Integer;
begin
  Result := 0;
  if TPWindow(Item1)^.Y>TPWindow(Item2)^.Y then Result := 1 else
  if TPWindow(Item1)^.Y<TPWindow(Item2)^.Y then Result := -1;
end;

procedure TWindowList.Fill(P:HWND);
var i,x,n : integer;
begin
  ClearAll;
  EnumChildWindows(P,@EnumWindowProc,Integer(Self));
  Sort(@SortFnX);
  n := 1;
  x := 0;
  for i:= 0 to Count-1 do
  begin
    if TPWindow(Items[i])^.X<>x then
    begin
      n := i+1;
      x := TPWindow(Items[i])^.X;
    end;
    TPWindow(Items[i])^.PosX := n;
  end;
  Sort(@SortFnY);
  n := 1;
  x := 0;
  for i:= 0 to Count-1 do
  begin
    if TPWindow(Items[i])^.Y<>x then
    begin
      n := i+1;
      x := TPWindow(Items[i])^.Y;
    end;
    TPWindow(Items[i])^.PosY := n;
  end;
end;


function EnumWindowProcClass(WinHandle: HWnd; Param: LongInt): Boolean; stdcall;
var
  i : integer;
  W : TWindow;
  Buf : array[1..200] of char;
begin
  Result := true;
  Buf[1] := #0;
  i := GetClassName(WinHandle,@Buf[1],200);
  if i>0 then W.ClassName := StrPas(@Buf[1]);
  if AnsiUpperCase(W.ClassName)=THwndList(Param).EnumClassName
  then THwndList(Param).Add(pointer(WinHandle));
end;

procedure THwndList.FillClass(CN:string);
begin
  Clear;
  EnumClassName := AnsiUpperCase(CN);
  EnumWindows(@EnumWindowProcClass,Integer(Self));
end;

end.
