unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ShellAPI, Menus, StdCtrls, Buttons, ExtCtrls, Grids, Data, ComCtrls;

type
  TMainForm = class(TForm)
    PopupMenu: TPopupMenu;
    miExit: TMenuItem;
    Timer: TTimer;
    miAuto: TMenuItem;
    miSettings: TMenuItem;
    Panel4: TPanel;
    btnAdd: TBitBtn;
    btnDel: TBitBtn;
    btnSave: TBitBtn;
    btnCancel: TBitBtn;
    lblFio: TLabel;
    lblEmail: TLabel;
    lblUrl: TLabel;
    miDisable: TMenuItem;
    Image: TImage;
    btnEdit: TBitBtn;
    Pages: TPageControl;
    tsLogin: TTabSheet;
    tsWindow: TTabSheet;
    Grid: TStringGrid;
    Panel1: TPanel;
    lblServer: TLabel;
    Panel2: TPanel;
    lblLogin: TLabel;
    Panel3: TPanel;
    lblWindows: TLabel;
    Windows: TStringGrid;
    ImageClick: TImage;
    TimerClick: TTimer;
    Panel5: TPanel;
    lblAuto: TLabel;
    btnAuto: TBitBtn;
    procedure WndProcNew(var Msg: TMessage);
    procedure AppMinimize(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ReadSettings;
    procedure WriteSettings;
    procedure SetLang(Rus:boolean);
    procedure ShowTrayIcon(IconShow : integer);
    procedure ShowMainForm;
    procedure ShowPasswordList;
    procedure ShowLoginWindows;
    procedure miExitClick(Sender: TObject);
    procedure EnterLogin(ClickOK:boolean);
    procedure EnterLoginWiindow(L:TLoginWindow;ClickOK:boolean);
    procedure btnOKlClick(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure miPassClick(Sender: TObject);
    procedure miAutoClick(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormActivate(Sender: TObject);
    procedure btnDelClick(Sender: TObject);
    procedure btnAddClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure LabelRed(Sender: TObject; Shift: TShiftState; X,Y: Integer);
    procedure LabelBlue(Sender: TObject; Shift: TShiftState; X,Y: Integer);
    procedure lblEmailClick(Sender: TObject);
    procedure lblUrlClick(Sender: TObject);
    procedure miDisableClick(Sender: TObject);
    procedure btnEditClick(Sender: TObject);
    procedure PagesChange(Sender: TObject);
    procedure TimerClickTimer(Sender: TObject);
    procedure btnAutoClick(Sender: TObject);
  public
    PassList     : TPassList;
    LoginList    : TLoginList;
    RusLang      : boolean;
  private
    FPrevWndProc : Pointer;
    FNewWndProc  : Pointer;
    IconVisible  : boolean;
    ShowClick    : boolean;
    TrayIcon     : TNotifyIconData;
    EnableLogin  : boolean;
    AutoLogin    : boolean;
    ShutDown     : boolean;
    FirstTime    : boolean;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses IniFiles, edtpass, wndlist, wndedit;

const
  CM_TRAYICON = WM_USER+10000;
  CL_NORMAL    = clBlack;
  CL_HIGHLIGHT = clRed;

procedure TMainForm.WndProcNew(var Msg: TMessage);
var
  P : TPoint;
begin
  try
    with Msg do
    begin
      if Msg = WM_QUERYENDSESSION then
      begin
        ShutDown := true;
        Result := 1;
        Exit;
      end;

      if Msg = CM_TRAYICON then
      begin
        if lParam = WM_LBUTTONDBLCLK then miDisableClick(nil);

        if lParam = WM_RBUTTONUP then
        begin
          GetCursorPos(P);
          SetForegroundWindow(Handle);
          try
            PopupMenu.Popup(P.X, P.Y);
          finally
            SetForegroundWindow(Handle);
          end;
        end;
      end;
      Result := CallWindowProc(FPrevWndProc, Handle, Msg, WParam, LParam);
    end;
  except
    Application.HandleException(Self);
  end;
end;

procedure TMainForm.AppMinimize(Sender: TObject);
begin
  ShowWindow(Handle,sw_hide);
  ShowWindow(Application.Handle,sw_hide);
end;


procedure TMainForm.SetLang(Rus:boolean);
begin
  if Rus then
  begin
    Caption := '';
    tsLogin.Caption    := '';
    tsWindow.Caption   := ' ';
    miDisable.Caption  := '';
    miAuto.Caption     := '';
    miSettings.Caption := '';
    miExit.Caption     := '';
    lblServer.Caption  := '';
    lblLogin.Caption   := '';
    lblAuto.Caption    := '';
    btnAdd.Caption     := '';
    btnEdit.Caption    := '';
    btnDel.Caption     := '';
    btnSave.Caption    := '';
    btnAuto.Caption    := '';
    btnCancel.Caption  := '';
    lblWindows.Caption := '  ';
    lblFio.Caption     := '(c)  ';
  end else
  begin
    Caption := 'Settings';
    tsLogin.Caption    := 'Passwords';
    tsWindow.Caption   := 'Login windows';
    miDisable.Caption  := 'Disable';
    miAuto.Caption     := 'Auto Login';
    miSettings.Caption := 'Settings';
    miExit.Caption     := 'Exit';
    lblServer.Caption  := 'Server';
    lblLogin.Caption   := 'Login';
    lblAuto.Caption    := 'Auto Login';
    btnAdd.Caption     := 'Add';
    btnEdit.Caption    := 'Edit';
    btnDel.Caption     := 'Delete';
    btnSave.Caption    := 'Save';
    btnAuto.Caption    := 'Auto Login';
    btnCancel.Caption  := 'Cancel';
    lblWindows.Caption := 'Login windows';
    lblFio.Caption     := '(c) Alex Boiko';
  end;
end;

procedure TMainForm.ReadSettings;
begin
  ReadKeyBool(nil,'','AutoLogin',AutoLogin,false);
end;


procedure TMainForm.WriteSettings;
begin
  WriteKeyBool(nil,'','AutoLogin',AutoLogin);
end;


procedure TMainForm.FormCreate(Sender: TObject);
begin
  Application.OnMinimize := AppMinimize;
  FNewWndProc := MakeObjectInstance(WndProcNew);
  FPrevWndProc := Pointer(GetWindowLong(Handle, GWL_WNDPROC));
  SetWindowLong(Handle, GWL_WNDPROC, LongInt(FNewWndProc));
  RusLang := (GetUserDefaultLangID mod 256)=LANG_RUSSIAN;
  SetLang(RusLang);
  Pages.ActivePage := tsLogin;
  IconVisible := false;

  PassList := TPassList.Create;
  LoginList := TLoginList.Create;
  PassList.Load;
  LoginList.Load;
  ShowPasswordList;
  ShowLoginWindows;
  ShutDown  := false;
  FirstTime := true;
  EnableLogin := true;
  ReadSettings;
  miAuto.Checked := AutoLogin;
end;


procedure TMainForm.FormDestroy(Sender: TObject);
begin
  SetWindowLong(Handle, GWL_WNDPROC, LongInt(FPrevWndProc));
  FreeObjectInstance(FNewWndProc);
  ShowTrayIcon(0);
  PassList.Free;
  LoginList.Free;
end;

procedure TMainForm.ShowTrayIcon(IconShow : integer);
begin
  with TrayIcon do
  begin
    cbSize := SizeOf(TNotifyIconData);
    Wnd := Handle;
    uFlags := NIF_MESSAGE or NIF_ICON or NIF_TIP;
    hIcon := Application.Icon.Handle;
    uCallbackMessage := CM_TRAYICON;
    uID := 0;
    if AutoLogin then
    begin
      if RusLang
      then StrPLCopy(szTip, ' ', SizeOf(szTip) - 1)
      else StrPLCopy(szTip, 'AutoLogin enabled', SizeOf(szTip) - 1);
    end else
    begin
      if RusLang
      then StrPLCopy(szTip, ' ', SizeOf(szTip) - 1)
      else StrPLCopy(szTip, 'AutoLogin disabled', SizeOf(szTip) - 1);
    end;
  end;

  if IconShow>0 then
  begin
    if IconShow=1 then
    begin
      if ShowClick
      then TrayIcon.hIcon := ImageClick.Picture.Icon.Handle
      else TrayIcon.hIcon := Application.Icon.Handle;
    end else TrayIcon.hIcon := Image.Picture.Icon.Handle;

    if not IconVisible
    then Shell_NotifyIcon(NIM_ADD, @TrayIcon)
    else Shell_NotifyIcon(NIM_MODIFY, @TrayIcon);

    IconVisible := true;
  end else
  begin
    if not IconVisible then Exit;
    Shell_NotifyIcon(NIM_DELETE, @TrayIcon);
    IconVisible := false;
  end;

end;


procedure TMainForm.ShowMainForm;
begin
  ShowWindow(Handle, SW_SHOW);
  Application.Restore;
  Application.BringToFront;
  Show;
end;


procedure TMainForm.ShowPasswordList;
var i : integer;
begin
  PassList.Sort(@SortFn);
  for i:=0 to Grid.RowCount-1 do
  begin
    Grid.Cells[0,i] := '';
    Grid.Cells[1,i] := '';
  end;
  Grid.RowCount := PassList.Count;
  for i:=0 to PassList.Count-1 do
  begin
    Grid.Cells[0,i] := ' '+TPPass(PassList[i])^.connect;
    Grid.Cells[1,i] := ' '+TPPass(PassList[i])^.username;
  end;
end;

procedure TMainForm.ShowLoginWindows;
var
  i : integer;
  yes : string;
begin
  LoginList.Sort(@SortFnWnd);
  for i:=0 to Windows.RowCount-1 do
  begin
    Windows.Cells[0,i] := '';
    Windows.Cells[1,i] := '';
  end;
  Windows.RowCount := LoginList.Count;
  if RusLang then yes := '' else yes := 'Yes';
  for i:=0 to LoginList.Count-1 do
  begin
    Windows.Cells[0,i] := ' '+TPLogin(LoginList[i])^.Name;
    if TPLogin(LoginList[i])^.AutoLogin then
    Windows.Cells[1,i] := ' '+yes;
  end;
end;

procedure TMainForm.miExitClick(Sender: TObject);
begin
  Application.Terminate;
end;


procedure TMainForm.EnterLogin(ClickOK:boolean);
var i : integer;
begin
  if not EnableLogin then Exit;
  for i := 0 to LoginList.Count-1 do
  EnterLoginWiindow(TPLogin(LoginList[i])^,
                    ClickOK and TPLogin(LoginList[i])^.AutoLogin);
end;


function EnumLoginWindowProc(WinHandle: HWnd; Param: LongInt): Boolean; stdcall;
begin
  Result := true;
  TList(param).Add(pointer(WinHandle));
end;


procedure TMainForm.EnterLoginWiindow(L:TLoginWindow;ClickOK:boolean);
var
  H : HWND;
  n : integer;
  WL : TWindowList;
  i : integer;
  j : TLoginWindowControl;
  A : array[TLoginWindowControl] of integer;
  k : shortint;
  Buf : array[1..500] of char;
  psw : string;
  hl : THwndList;
  res : longint;

function Match(W:TWindow;C:TLoginControl) : boolean;
begin
  Result := false;
  if W.ClassName<>C.ClassName then Exit;
  if (W.PosX<>C.PosX) and (C.PosX<>0) then Exit;
  if (W.PosY<>C.PosY) and (C.PosY<>0) then Exit;
  if (C.Text<>'') and (C.Text<>W.Text) then Exit;
  Result := true;
end;


begin
  HL := THwndList.Create;
  WL := TWindowList.Create;
  HL.FillClass(L.ClassName);
  for n := 0 to HL.Count-1 do
  begin
    H := HWND(HL[n]);

    WL.ClearAll;
    WL.Fill(H);

    for i := 0 to WL.Count-1 do
    begin
      for j := Low(TLoginWindowControl) to High(TLoginWindowControl) do
      begin
        if Match(TPWindow(WL[i])^,L.Controls[j])
        then TPWindow(WL[i])^.Kind := Ord(j);
      end;
    end;

    for j := Low(TLoginWindowControl) to High(TLoginWindowControl) do A[j] := -1;

    for i := 0 to WL.Count-1 do
    begin
      k := TPWindow(WL[i])^.Kind;
      if (k>=0) and (A[TLoginWindowControl(k)]<0) then A[TLoginWindowControl(k)] := i;
    end;

    if (A[Server]>=0) and (A[Login]>=0) and (A[Password]>=0) then
    begin
      PassList.Password(TPWindow(WL[A[Login]])^.Text,
                        TPWindow(WL[A[Server]])^.Text,psw);
      if psw='' then continue;

      Buf[1] := #0;
      StrPCopy(@Buf[1],psw);
      if IsWindowEnabled(TPWindow(WL[A[Password]])^.hWnd) then
      begin
        TimerClick.Enabled := true;
        ShowClick := true;
        ShowTrayIcon(1);
        SendMessageTimeout(TPWindow(WL[A[Password]])^.hWnd,WM_SETTEXT,
                           0,integer(@Buf[1]),SMTO_ABORTIFHUNG,5,res);
      end;
      if (A[Connect]>=0) and ClickOK  then
      begin
        if IsWindowEnabled(TPWindow(WL[A[Connect]])^.hWnd) then
        begin
          PostMessage(TPWindow(WL[A[Connect]])^.hWnd, WM_LBUTTONDOWN, 0, $00010001);
          PostMessage(TPWindow(WL[A[Connect]])^.hWnd, WM_LBUTTONUP, 0, $00010001);
        end;
      end;
    end;
  end;
  HL.Free;
  WL.Free;
end;


procedure TMainForm.btnOKlClick(Sender: TObject);
begin
  PassList.Save;
  LoginList.Save;
  ShowPasswordList;
  ShowLoginWindows;
  Hide;
  Application.Minimize;
end;

procedure TMainForm.TimerTimer(Sender: TObject);
begin
  EnterLogin(AutoLogin);
  lblEmail.Font.Color := CL_NORMAL;
  lblUrl.Font.Color := CL_NORMAL;
end;

procedure TMainForm.btnSaveClick(Sender: TObject);
begin
  PassList.Save;
  PassList.Load;
end;

procedure TMainForm.miPassClick(Sender: TObject);
begin
  ShowMainForm;
end;

procedure TMainForm.miAutoClick(Sender: TObject);
begin
  AutoLogin := not AutoLogin;
  miAuto.Checked := AutoLogin;
  WriteSettings;
  ShowTrayIcon(1);
end;


procedure TMainForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #27 then Application.Minimize;
end;

procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if ShutDown then CanClose := true else
  begin
    CanClose := false;
    Application.Minimize;
  end;
end;

procedure TMainForm.FormActivate(Sender: TObject);
begin
  if FirstTime then
  begin
    Application.Minimize;
    ShowWindow(Handle,sw_hide);
    ShowWindow(Application.Handle,sw_hide);
    FirstTime := false;
  end;
end;

procedure TMainForm.btnDelClick(Sender: TObject);
begin
  if Pages.ActivePage=tsLogin then
  begin
    if Grid.Row<PassList.Count then
    begin
      PassList.Delete(Grid.Row);
      ShowPasswordList;
    end;
  end;

  if Pages.ActivePage=tsWindow then
  begin
    if Windows.Row<LoginList.Count then
    begin
      LoginList.Delete(Windows.Row);
      ShowLoginWindows;
    end;
  end;
end;


procedure TMainForm.btnAddClick(Sender: TObject);
var
  R : TPass;
  H : integer;
  W : TLoginWindow;
begin
  if Pages.ActivePage=tsLogin then
  begin
    FillChar(R,SizeOf(TPass),0);
    if EditPass(R) then
    begin
      PassList.AddLogin(R);
      ShowPasswordList;
    end;
  end;

  if Pages.ActivePage=tsWindow then
  begin
    H := SelectWindow(GetDesktopWindow);
    if H=0 then Exit;
    if EditWindow(H,W) then
    begin
      LoginList.AddLogin(W);
      ShowLoginWindows;
    end;
  end;
end;


procedure TMainForm.btnEditClick(Sender: TObject);
var
  i : integer;
  R : TPass;
begin
  i := Grid.Row;
  if i<PassList.Count then
  begin
    R := TPPass(PassList[i])^;
    if EditPass(R) then
    begin
      TPPass(PassList[i])^ := R;
      ShowPasswordList;
    end;
  end;
end;


procedure TMainForm.btnCancelClick(Sender: TObject);
begin
  PassList.Load;
  ShowPasswordList;
  Hide;
  Application.Minimize;
end;


procedure TMainForm.LabelRed(Sender: TObject; Shift: TShiftState; X,Y: Integer);
begin
  TLabel(Sender).Font.Color := CL_HIGHLIGHT;
  if Sender=lblEmail
  then lblUrl.Font.Color := CL_NORMAL
  else lblEmail.Font.Color := CL_NORMAL;
end;


procedure TMainForm.LabelBlue(Sender: TObject; Shift: TShiftState; X,Y: Integer);
begin
  lblEmail.Font.Color := CL_NORMAL;
  lblUrl.Font.Color := CL_NORMAL;
end;


procedure TMainForm.lblEmailClick(Sender: TObject);
begin
  ShellExecute(Handle,'open','mailto:alexboiko@mtu-net.ru','','',SW_SHOWNORMAL);
end;


procedure TMainForm.lblUrlClick(Sender: TObject);
begin
  ShellExecute(Handle,'open','http://alexboiko.da.ru','','',SW_SHOWNORMAL);
end;


procedure TMainForm.miDisableClick(Sender: TObject);
begin
  EnableLogin := not EnableLogin;
  if EnableLogin then
  begin
    if RusLang
    then miDisable.Caption := ''
    else miDisable.Caption := 'Disable';
    miAuto.Enabled := true;
    ShowTrayIcon(1);
  end else
  begin
    if RusLang
    then miDisable.Caption := ''
    else miDisable.Caption := 'Enable';
    miAuto.Enabled := false;
    ShowTrayIcon(2);
  end;
end;


procedure TMainForm.PagesChange(Sender: TObject);
begin
  if Pages.ActivePage = tsWindow then
  begin
    btnEdit.Visible := false;
    btnAuto.Visible := true;
    btnDel.Top      := btnEdit.Top;
  end else
  begin
    btnEdit.Enabled := true;
    btnEdit.Visible := true;
    btnAuto.Visible := false;
    btnDel.Top      := btnEdit.Top + (btnEdit.Top-btnAdd.Top);
  end;
end;


procedure TMainForm.TimerClickTimer(Sender: TObject);
begin
  TimerClick.Enabled := false;
  ShowClick := false;
  ShowTrayIcon(1);
end;



procedure TMainForm.btnAutoClick(Sender: TObject);
begin
  TPLogin(LoginList.Items[Windows.Row])^.AutoLogin :=
    not TPLogin(LoginList.Items[Windows.Row])^.AutoLogin;
  ShowLoginWindows;
end;

end.
